﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace University_Code_Stage_I
{
    public class Subject
    {
        public string Name { get; set; }

        public List<Student> activeStudents = new List<Student>();

        // ZH és vizsgaeredmények
        private List<Midterm> midTermRequirements;
        private List<MidtermResult> midtermResults = new List<MidtermResult>();
        private List<ExamResult> examResults = new List<ExamResult>();

        public Subject(string name, List<Midterm> midTermRequirements)
        {
            this.Name = name;
            this.midTermRequirements = midTermRequirements;
        }

        public void AddActiveStudent(Student st)
        {
            if (!activeStudents.Contains(st))
                activeStudents.Add(st);
        }

        public void RemoveActiveStudent(Student st)
        {
            if (activeStudents.Contains(st))
                activeStudents.Remove(st);

            // A félévközi eredményeket töröljük, kivéve akkor, ha meglett az aláírás, de a vizsga nem
            if (!(IsMidtermRequirementsFinished(st) && !IsSubjectFinished(st)))
            {
                Console.WriteLine("###" + this.Name + "### Removing " + st.Name + "'s midterm results ");
                midtermResults.RemoveAll(mr => mr.student == st);
            }
        }

        public void AddMidtermResult(Student st, Midterm m, bool result)
        {
            // Először ellenőrizzük, hogy az adott ZH és hallgató ehhez a tárgyhoz tartozik-e
            if (!midTermRequirements.Any(mr => mr == m) || !activeStudents.Contains(st))
                return;

            // Megnézzük, hogy volt-e már az adott ZH-n valamilyen eredménye
            var previousResult = midtermResults.FirstOrDefault(mr => mr.student == st && mr.midterm == m);

            // Ha nem volt, hozzáadjuk a ZH eredményét (sikerült vagy bukott) az eredményekhez
            if (previousResult == null)
            {
                midtermResults.Add(new MidtermResult() { student = st, midterm = m, success = result });
                Console.WriteLine("###" + this.Name + "### Added midterm '" + m.Name + "' result (" + result.ToString() + ")"
                     + " to " + st.Name);
            }

            // Ha volt, akkor átállítjuk a megfelelő státuszra, de csak akkor, ha még pót ZH-zhat
            // Ekkor beállítjuk, hogy pót ZH volt
            // MEGJEGYZÉS: jelenleg ronthat is a pót ZH-n, vagyis bukhatja a ZH-t
            else if (!previousResult.isRepeatMidterm)
            {
                previousResult.success = result;
                previousResult.isRepeatMidterm = true;
                Console.WriteLine("###" + this.Name + "### Added repeated midterm '" + m.Name + "' result (" + result.ToString() + ")"
                     + " to " + st.Name);
            }
        }

        public void AddExamResult(Student st, bool result)
        {
            // Először megnézzük, hogy a tárgyhoz tartozik-e, és van-e aláírása a hallgatónak, vagyis sikerültek-e a ZH-i
            if (!activeStudents.Contains(st) || !IsMidtermRequirementsFinished(st))
                return;

            // Ha sikerültek a ZH-k, akkor először megnézzük, hogy volt-e már a vizsgán eredménye
            var previousResult = examResults.FirstOrDefault(er => er.student == st);

            // Ha volt, akkor az eredmény marad, egyébként pedig beállítjuk a most kapott eredményt
            if (previousResult == null)
            {
                examResults.Add(new ExamResult() { student = st, success = result });
                Console.WriteLine("###" + this.Name + "### Added exam result (" + result.ToString() + ")"
                     + " to " + st.Name);
            }
        }

        public bool IsMidtermRequirementsFinished(Student st)
        {
            // Először megkeressük a hallgató által teljesített ZH-kat
            var finishedMidterms = midtermResults.Where(mr => mr.student == st && mr.success == true).ToList();

            // Majd megnézzük, hogy minden ZH-t teljesített-e, ami a követelmények között szerepel
            return midTermRequirements.All(
                mr => finishedMidterms.Any(
                    fm => fm.midterm == mr && fm.success == true
                    )
                );
        }

        public bool IsSubjectFinished(Student st)
        {
            // Ha nincs vizsgaeredmény (nem jutott el addig), vagy nem sikerült a vizsga, akkor bukta a tárgyat
            return examResults.Any(er => er.student == st && er.success == true);
        }
    }
}
